@@ -85,7 +85,7 @@
 # define STDERR_FILENO   2
 #endif
 
-#if defined(HAVE_DECL_O_NONBLOCK) && HAVE_DECL_O_NONBLOCK == 0
+#if !HAVE_DECL_O_NONBLOCK
 # define O_NONBLOCK      00004	/* Non Blocking Open */
 #endif
 
@@ -422,15 +422,6 @@ typedef uint16_t	in_port_t;
 #define INET6_ADDRSTRLEN 46
 #endif
 
-/*
- * Platforms that have arc4random_uniform() and not arc4random_stir()
- * shouldn't need the latter.
- */
-#if defined(HAVE_ARC4RANDOM) && defined(HAVE_ARC4RANDOM_UNIFORM) && \
-    !defined(HAVE_ARC4RANDOM_STIR)
-# define arc4random_stir()
-#endif
-
 #ifndef HAVE_VA_COPY
 # ifdef HAVE___VA_COPY
 #  define va_copy(dest, src) __va_copy(dest, src)
@@ -497,10 +488,6 @@ typedef uint16_t	in_port_t;
 #define FPARSELN_UNESCALL       0x0f
 #endif
 
-#ifndef _PATH_VARRUN
-#define _PATH_VARRUN "/var/run/"
-#endif
-
 #ifdef HAVE_M_DATA
 #undef M_DATA
 #endif
@@ -516,4 +503,12 @@ typedef uint16_t	in_port_t;
 #define LOCK_UN         0x08            /* unlock file */
 #endif
 
+#if !HAVE_DECL_LOG_PERROR
+#define LOG_PERROR 0
+#endif
+
+#if !HAVE_DECL_SSL_OP_NO_CLIENT_RENEGOTIATION
+#define SSL_OP_NO_CLIENT_RENEGOTIATION 0
+#endif
+
 #endif /* _DEFINES_H */
