@@ -1,4 +1,4 @@
-/*	$OpenBSD: ca.c,v 1.22 2016/05/28 21:21:20 eric Exp $	*/
+/*	$OpenBSD: ca.c,v 1.28 2017/11/21 12:20:34 eric Exp $	*/
 
 /*
  * Copyright (c) 2014 Reyk Floeter <reyk@openbsd.org>
@@ -69,29 +69,14 @@ static uint64_t	 rsae_reqid = 0;
 static void
 ca_shutdown(void)
 {
-	log_info("info: ca agent exiting");
+	log_debug("debug: ca agent exiting");
 	_exit(0);
 }
 
-static void
-ca_sig_handler(int sig, short event, void *p)
-{
-	switch (sig) {
-	case SIGINT:
-	case SIGTERM:
-		ca_shutdown();
-		break;
-	default:
-		fatalx("ca_sig_handler: unexpected signal");
-	}
-}
-
 int
 ca(void)
 {
 	struct passwd	*pw;
-	struct event	 ev_sigint;
-	struct event	 ev_sigterm;
 
 	purge_config(PURGE_LISTENERS|PURGE_TABLES|PURGE_RULES);
 
@@ -113,17 +98,14 @@ ca(void)
 	imsg_callback = ca_imsg;
 	event_init();
 
-	signal_set(&ev_sigint, SIGINT, ca_sig_handler, NULL);
-	signal_set(&ev_sigterm, SIGTERM, ca_sig_handler, NULL);
-	signal_add(&ev_sigint, NULL);
-	signal_add(&ev_sigterm, NULL);
+	signal(SIGINT, SIG_IGN);
+	signal(SIGTERM, SIG_IGN);
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGHUP, SIG_IGN);
 
 	config_peer(PROC_CONTROL);
 	config_peer(PROC_PARENT);
 	config_peer(PROC_PONY);
-	config_done();
 
 	/* Ignore them until we get our config */
 	mproc_disable(p_pony);
@@ -131,9 +113,8 @@ ca(void)
 	if (pledge("stdio", NULL) == -1)
 		err(1, "pledge");
 
-	if (event_dispatch() < 0)
-		fatal("event_dispatch");
-	ca_shutdown();
+	event_dispatch();
+	fatalx("exited event loop");
 
 	return (0);
 }
@@ -164,8 +145,7 @@ ca_init(void)
 
 		pki->pki_pkey = pkey;
 
-		explicit_bzero(pki->pki_key, pki->pki_key_len);
-		free(pki->pki_key);
+		freezero(pki->pki_key, pki->pki_key_len);
 		pki->pki_key = NULL;
 	}
 }
@@ -226,10 +206,8 @@ end:
 			*errstr = ERR_error_string(ERR_peek_last_error(), NULL);
 	}
 
-	if (xsc)
-		X509_STORE_CTX_free(xsc);
-	if (store)
-		X509_STORE_free(store);
+	X509_STORE_CTX_free(xsc);
+	X509_STORE_free(store);
 
 	return ret > 0 ? 1 : 0;
 }
@@ -248,79 +226,73 @@ ca_imsg(struct mproc *p, struct imsg *imsg)
 	uint64_t		 id;
 	int			 v;
 
-	if (p->proc == PROC_PARENT) {
-		switch (imsg->hdr.type) {
-		case IMSG_CONF_START:
-			return;
-		case IMSG_CONF_END:
-			ca_init();
+	if (imsg == NULL)
+		ca_shutdown();
 
-			/* Start fulfilling requests */
-			mproc_enable(p_pony);
-			return;
-		}
-	}
+	switch (imsg->hdr.type) {
+	case IMSG_CONF_START:
+		return;
+	case IMSG_CONF_END:
+		ca_init();
 
-	if (p->proc == PROC_CONTROL) {
-		switch (imsg->hdr.type) {
-		case IMSG_CTL_VERBOSE:
-			m_msg(&m, imsg);
-			m_get_int(&m, &v);
-			m_end(&m);
-			log_verbose(v);
-			return;
-		case IMSG_CTL_PROFILE:
-			m_msg(&m, imsg);
-			m_get_int(&m, &v);
-			m_end(&m);
-			profiling = v;
-			return;
-		}
-	}
+		/* Start fulfilling requests */
+		mproc_enable(p_pony);
+		return;
+
+	case IMSG_CTL_VERBOSE:
+		m_msg(&m, imsg);
+		m_get_int(&m, &v);
+		m_end(&m);
+		log_trace_verbose(v);
+		return;
+
+	case IMSG_CTL_PROFILE:
+		m_msg(&m, imsg);
+		m_get_int(&m, &v);
+		m_end(&m);
+		profiling = v;
+		return;
+
+	case IMSG_CA_PRIVENC:
+	case IMSG_CA_PRIVDEC:
+		m_msg(&m, imsg);
+		m_get_id(&m, &id);
+		m_get_string(&m, &pkiname);
+		m_get_data(&m, &from, &flen);
+		m_get_size(&m, &tlen);
+		m_get_size(&m, &padding);
+		m_end(&m);
+
+		pki = dict_get(env->sc_pki_dict, pkiname);
+		if (pki == NULL || pki->pki_pkey == NULL ||
+		    (rsa = EVP_PKEY_get1_RSA(pki->pki_pkey)) == NULL)
+			fatalx("ca_imsg: invalid pki");
+
+		if ((to = calloc(1, tlen)) == NULL)
+			fatalx("ca_imsg: calloc");
 
-	if (p->proc == PROC_PONY) {
 		switch (imsg->hdr.type) {
 		case IMSG_CA_PRIVENC:
+			ret = RSA_private_encrypt(flen, from, to, rsa,
+			    padding);
+			break;
 		case IMSG_CA_PRIVDEC:
-			m_msg(&m, imsg);
-			m_get_id(&m, &id);
-			m_get_string(&m, &pkiname);
-			m_get_data(&m, &from, &flen);
-			m_get_size(&m, &tlen);
-			m_get_size(&m, &padding);
-			m_end(&m);
-
-			pki = dict_get(env->sc_pki_dict, pkiname);
-			if (pki == NULL || pki->pki_pkey == NULL ||
-			    (rsa = EVP_PKEY_get1_RSA(pki->pki_pkey)) == NULL)
-				fatalx("ca_imsg: invalid pki");
-
-			if ((to = calloc(1, tlen)) == NULL)
-				fatalx("ca_imsg: calloc");
-
-			switch (imsg->hdr.type) {
-			case IMSG_CA_PRIVENC:
-				ret = RSA_private_encrypt(flen, from, to, rsa,
-				    padding);
-				break;
-			case IMSG_CA_PRIVDEC:
-				ret = RSA_private_decrypt(flen, from, to, rsa,
-				    padding);
-				break;
-			}
-
-			m_create(p, imsg->hdr.type, 0, 0, -1);
-			m_add_id(p, id);
-			m_add_int(p, ret);
-			if (ret > 0)
-				m_add_data(p, to, (size_t)ret);
-			m_close(p);
-
-			free(to);
-			RSA_free(rsa);
-
-			return;
+			ret = RSA_private_decrypt(flen, from, to, rsa,
+			    padding);
+			break;
 		}
+
+		m_create(p, imsg->hdr.type, 0, 0, -1);
+		m_add_id(p, id);
+		m_add_int(p, ret);
+		if (ret > 0)
+			m_add_data(p, to, (size_t)ret);
+		m_close(p);
+
+		free(to);
+		RSA_free(rsa);
+
+		return;
 	}
 
 	errx(1, "ca_imsg: unexpected %s imsg", imsg_to_str(imsg->hdr.type));
