@@ -1,4 +1,4 @@
-/*	$OpenBSD: control.c,v 1.113 2016/05/28 21:21:20 eric Exp $	*/
+/*	$OpenBSD: control.c,v 1.120 2017/11/27 08:35:59 sunil Exp $	*/
 
 /*
  * Copyright (c) 2012 Gilles Chehade <gilles@poolp.org>
@@ -66,7 +66,6 @@ static void control_shutdown(void);
 static void control_listen(void);
 static void control_accept(int, short, void *);
 static void control_close(struct ctl_conn *);
-static void control_sig_handler(int, short, void *);
 static void control_dispatch_ext(struct mproc *, struct imsg *);
 static void control_digest_update(const char *, size_t, int);
 static void control_broadcast_verbose(int, int);
@@ -92,62 +91,39 @@ control_imsg(struct mproc *p, struct imsg *imsg)
 	const void		*data;
 	size_t			 sz;
 
-	if (p->proc == PROC_PONY) {
-		switch (imsg->hdr.type) {
-		case IMSG_CTL_SMTP_SESSION:
-			c = tree_get(&ctl_conns, imsg->hdr.peerid);
-			if (c == NULL)
-				return;
-			m_compose(&c->mproc, IMSG_CTL_OK, 0, 0, imsg->fd,
-			    NULL, 0);
-			return;
-		}
-	}
-	if (p->proc == PROC_SCHEDULER) {
-		switch (imsg->hdr.type) {
-		case IMSG_CTL_OK:
-		case IMSG_CTL_FAIL:
-		case IMSG_CTL_LIST_MESSAGES:
-			c = tree_get(&ctl_conns, imsg->hdr.peerid);
-			if (c == NULL)
-				return;
-			imsg->hdr.peerid = 0;
-			m_forward(&c->mproc, imsg);
-			return;
-		}
-	}
-	if (p->proc == PROC_QUEUE) {
-		switch (imsg->hdr.type) {
-		case IMSG_CTL_LIST_ENVELOPES:
-		case IMSG_CTL_DISCOVER_EVPID:
-		case IMSG_CTL_DISCOVER_MSGID:
-		case IMSG_CTL_UNCORRUPT_MSGID:
-			c = tree_get(&ctl_conns, imsg->hdr.peerid);
-			if (c == NULL)
-				return;
-			m_forward(&c->mproc, imsg);
-			return;
-		}
-	}
-	if (p->proc == PROC_PONY) {
-		switch (imsg->hdr.type) {
-		case IMSG_CTL_OK:
-		case IMSG_CTL_FAIL:
-		case IMSG_CTL_MTA_SHOW_HOSTS:
-		case IMSG_CTL_MTA_SHOW_RELAYS:
-		case IMSG_CTL_MTA_SHOW_ROUTES:
-		case IMSG_CTL_MTA_SHOW_HOSTSTATS:
-		case IMSG_CTL_MTA_SHOW_BLOCK:
-			c = tree_get(&ctl_conns, imsg->hdr.peerid);
-			if (c == NULL)
-				return;
-			imsg->hdr.peerid = 0;
-			m_forward(&c->mproc, imsg);
-			return;
-		}
+	if (imsg == NULL) {
+		if (p->proc != PROC_CLIENT)
+			control_shutdown();
+		return;
 	}
 
 	switch (imsg->hdr.type) {
+	case IMSG_CTL_OK:
+	case IMSG_CTL_FAIL:
+	case IMSG_CTL_LIST_MESSAGES:
+	case IMSG_CTL_LIST_ENVELOPES:
+	case IMSG_CTL_DISCOVER_EVPID:
+	case IMSG_CTL_DISCOVER_MSGID:
+	case IMSG_CTL_UNCORRUPT_MSGID:
+	case IMSG_CTL_MTA_SHOW_HOSTS:
+	case IMSG_CTL_MTA_SHOW_RELAYS:
+	case IMSG_CTL_MTA_SHOW_ROUTES:
+	case IMSG_CTL_MTA_SHOW_HOSTSTATS:
+	case IMSG_CTL_MTA_SHOW_BLOCK:
+		c = tree_get(&ctl_conns, imsg->hdr.peerid);
+		if (c == NULL)
+			return;
+		imsg->hdr.peerid = 0;
+		m_forward(&c->mproc, imsg);
+		return;
+
+	case IMSG_CTL_SMTP_SESSION:
+		c = tree_get(&ctl_conns, imsg->hdr.peerid);
+		if (c == NULL)
+			return;
+		m_compose(&c->mproc, IMSG_CTL_OK, 0, 0, imsg->fd, NULL, 0);
+		return;
+
 	case IMSG_STAT_INCREMENT:
 		m_msg(&m, imsg);
 		m_get_string(&m, &key);
@@ -160,6 +136,7 @@ control_imsg(struct mproc *p, struct imsg *imsg)
 			stat_backend->increment(key, val.u.counter);
 		control_digest_update(key, val.u.counter, 1);
 		return;
+
 	case IMSG_STAT_DECREMENT:
 		m_msg(&m, imsg);
 		m_get_string(&m, &key);
@@ -172,6 +149,7 @@ control_imsg(struct mproc *p, struct imsg *imsg)
 			stat_backend->decrement(key, val.u.counter);
 		control_digest_update(key, val.u.counter, 0);
 		return;
+
 	case IMSG_STAT_SET:
 		m_msg(&m, imsg);
 		m_get_string(&m, &key);
@@ -189,19 +167,6 @@ control_imsg(struct mproc *p, struct imsg *imsg)
 	    imsg_to_str(imsg->hdr.type));
 }
 
-static void
-control_sig_handler(int sig, short event, void *p)
-{
-	switch (sig) {
-	case SIGINT:
-	case SIGTERM:
-		control_shutdown();
-		break;
-	default:
-		fatalx("control_sig_handler: unexpected signal");
-	}
-}
-
 int
 control_create_socket(void)
 {
@@ -248,8 +213,6 @@ int
 control(void)
 {
 	struct passwd		*pw;
-	struct event		 ev_sigint;
-	struct event		 ev_sigterm;
 
 	purge_config(PURGE_EVERYTHING);
 
@@ -274,10 +237,8 @@ control(void)
 	imsg_callback = control_imsg;
 	event_init();
 
-	signal_set(&ev_sigint, SIGINT, control_sig_handler, NULL);
-	signal_set(&ev_sigterm, SIGTERM, control_sig_handler, NULL);
-	signal_add(&ev_sigint, NULL);
-	signal_add(&ev_sigterm, NULL);
+	signal(SIGINT, SIG_IGN);
+	signal(SIGTERM, SIG_IGN);
 	signal(SIGPIPE, SIG_IGN);
 	signal(SIGHUP, SIG_IGN);
 
@@ -293,16 +254,14 @@ control(void)
 	config_peer(PROC_LKA);
 	config_peer(PROC_PONY);
 	config_peer(PROC_CA);
-	config_done();
 
 	control_listen();
 
 	if (pledge("stdio unix recvfd sendfd", NULL) == -1)
 		err(1, "pledge");
 
-	if (event_dispatch() < 0)
-		fatal("event_dispatch");
-	control_shutdown();
+	event_dispatch();
+	fatalx("exited event loop");
 
 	return (0);
 }
@@ -310,7 +269,7 @@ control(void)
 static void
 control_shutdown(void)
 {
-	log_info("info: control process exiting");
+	log_debug("debug: control agent exiting");
 	_exit(0);
 }
 
@@ -489,7 +448,7 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 
 	switch (imsg->hdr.type) {
 	case IMSG_CTL_SMTP_SESSION:
-		if (env->sc_flags & (SMTPD_SMTP_PAUSED | SMTPD_EXITING)) {
+		if (env->sc_flags & SMTPD_SMTP_PAUSED) {
 			m_compose(p, IMSG_CTL_FAIL, 0, 0, -1, NULL, 0);
 			return;
 		}
@@ -517,34 +476,17 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		m_compose(p, IMSG_CTL_GET_STATS, 0, 0, -1, kvp, sizeof *kvp);
 		return;
 
-	case IMSG_CTL_SHUTDOWN:
-		/* NEEDS_FIX */
-		log_debug("debug: received shutdown request");
-
-		if (c->euid)
-			goto badcred;
-
-		if (env->sc_flags & SMTPD_EXITING) {
-			m_compose(p, IMSG_CTL_FAIL, 0, 0, -1, NULL, 0);
-			return;
-		}
-		env->sc_flags |= SMTPD_EXITING;
-		m_compose(p, IMSG_CTL_OK, 0, 0, -1, NULL, 0);
-		m_compose(p_parent, IMSG_CTL_SHUTDOWN, 0, 0, -1, NULL, 0);
-		return;
-
 	case IMSG_CTL_VERBOSE:
 		if (c->euid)
 			goto badcred;
 
-		if (imsg->hdr.len - IMSG_HEADER_SIZE != sizeof(verbose))
+		if (imsg->hdr.len - IMSG_HEADER_SIZE != sizeof(v))
 			goto badcred;
 
 		memcpy(&v, imsg->data, sizeof(v));
-		verbose = v;
-		log_verbose(verbose);
+		log_trace_verbose(v);
 
-		control_broadcast_verbose(IMSG_CTL_VERBOSE, verbose);
+		control_broadcast_verbose(IMSG_CTL_VERBOSE, v);
 
 		m_compose(p, IMSG_CTL_OK, 0, 0, -1, NULL, 0);
 		return;
@@ -553,14 +495,14 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		if (c->euid)
 			goto badcred;
 
-		if (imsg->hdr.len - IMSG_HEADER_SIZE != sizeof(verbose))
+		if (imsg->hdr.len - IMSG_HEADER_SIZE != sizeof(v))
 			goto badcred;
 
 		memcpy(&v, imsg->data, sizeof(v));
-		verbose |= v;
-		log_verbose(verbose);
+		tracing |= v;
+		log_trace_verbose(tracing);
 
-		control_broadcast_verbose(IMSG_CTL_VERBOSE, verbose);
+		control_broadcast_verbose(IMSG_CTL_VERBOSE, tracing);
 
 		m_compose(p, IMSG_CTL_OK, 0, 0, -1, NULL, 0);
 		return;
@@ -569,14 +511,14 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		if (c->euid)
 			goto badcred;
 
-		if (imsg->hdr.len - IMSG_HEADER_SIZE != sizeof(verbose))
+		if (imsg->hdr.len - IMSG_HEADER_SIZE != sizeof(v))
 			goto badcred;
 
 		memcpy(&v, imsg->data, sizeof(v));
-		verbose &= ~v;
-		log_verbose(verbose);
+		tracing &= ~v;
+		log_trace_verbose(tracing);
 
-		control_broadcast_verbose(IMSG_CTL_VERBOSE, verbose);
+		control_broadcast_verbose(IMSG_CTL_VERBOSE, tracing);
 
 		m_compose(p, IMSG_CTL_OK, 0, 0, -1, NULL, 0);
 		return;
@@ -585,7 +527,7 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		if (c->euid)
 			goto badcred;
 
-		if (imsg->hdr.len - IMSG_HEADER_SIZE != sizeof(verbose))
+		if (imsg->hdr.len - IMSG_HEADER_SIZE != sizeof(v))
 			goto badcred;
 
 		memcpy(&v, imsg->data, sizeof(v));
@@ -600,7 +542,7 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		if (c->euid)
 			goto badcred;
 
-		if (imsg->hdr.len - IMSG_HEADER_SIZE != sizeof(verbose))
+		if (imsg->hdr.len - IMSG_HEADER_SIZE != sizeof(v))
 			goto badcred;
 
 		memcpy(&v, imsg->data, sizeof(v));
@@ -792,8 +734,8 @@ control_dispatch_ext(struct mproc *p, struct imsg *imsg)
 		if (len >= LINE_MAX)
 			goto invalid;
 
+		imsg->hdr.peerid = c->id;
 		m_forward(p_lka, imsg);
-		m_compose(p, IMSG_CTL_OK, 0, 0, -1, NULL, 0);
 		return;
 
 	case IMSG_CTL_DISCOVER_EVPID:
