@@ -1,4 +1,4 @@
-/*	$OpenBSD: delivery_mbox.c,v 1.10 2014/04/19 11:31:06 gilles Exp $	*/
+/*	$OpenBSD: delivery_mbox.c,v 1.12 2015/12/22 07:54:57 sunil Exp $	*/
 
 /*
  * Copyright (c) 2011 Gilles Chehade <gilles@poolp.org>
@@ -28,7 +28,9 @@
 #include <event.h>
 #include <fcntl.h>
 #include <imsg.h>
+#ifdef HAVE_PATHS_H
 #include <paths.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -62,7 +64,8 @@ delivery_mbox_open(struct deliver *deliver)
 	environ = environ_new;
 
 	if (deliver->from[0] == '\0')
-		(void)strlcpy(deliver->from, "MAILER-DAEMON", sizeof deliver->from);
+		(void)strlcpy(deliver->from, "MAILER-DAEMON",
+		    sizeof deliver->from);
 	execle(PATH_MAILLOCAL, PATH_MAILLOCAL, "-f", deliver->from,
 	    deliver->to, (char *)NULL, environ_new);
 	perror("execle");
