@@ -1,4 +1,4 @@
-/*	$OpenBSD: mproc.c,v 1.20 2016/06/06 20:48:15 eric Exp $	*/
+/*	$OpenBSD: mproc.c,v 1.29 2017/03/17 20:57:57 eric Exp $	*/
 
 /*
  * Copyright (c) 2012 Eric Faurot <eric@faurot.net>
@@ -43,7 +43,6 @@
 
 static void mproc_dispatch(int, short, void *);
 
-static ssize_t msgbuf_write2(struct msgbuf *);
 static ssize_t imsg_read_nofd(struct imsgbuf *);
 
 int
@@ -90,6 +89,8 @@ mproc_init(struct mproc *p, int fd)
 void
 mproc_clear(struct mproc *p)
 {
+	log_debug("debug: clearing p=%s, fd=%d, pid=%d", p->name, p->imsgbuf.fd, p->pid);
+
 	event_del(&p->ev);
 	close(p->imsgbuf.fd);
 	imsg_clear(&p->imsgbuf);
@@ -168,31 +169,23 @@ mproc_dispatch(int fd, short event, void *arg)
 			/* NOTREACHED */
 		case 0:
 			/* this pipe is dead, so remove the event handler */
-			if (smtpd_process != PROC_CONTROL ||
-			    p->proc != PROC_CLIENT)
-				log_warnx("warn: %s -> %s: pipe closed",
-				    proc_name(smtpd_process),  p->name);
+			log_debug("debug: %s -> %s: pipe closed",
+			    proc_name(smtpd_process),  p->name);
 			p->handler(p, NULL);
 			return;
 		default:
-			p->bytes_in += n;
 			break;
 		}
 	}
 
 	if (event & EV_WRITE) {
-		n = msgbuf_write2(&p->imsgbuf.w);
+		n = msgbuf_write(&p->imsgbuf.w);
 		if (n == 0 || (n == -1 && errno != EAGAIN)) {
 			/* this pipe is dead, so remove the event handler */
-			if (smtpd_process != PROC_CONTROL ||
-			    p->proc != PROC_CLIENT)
-				log_warnx("warn: %s -> %s: pipe closed",
-				    proc_name(smtpd_process),  p->name);
+			log_debug("debug: %s -> %s: pipe closed",
+			    proc_name(smtpd_process),  p->name);
 			p->handler(p, NULL);
 			return;
-		} else if (n != -1) {
-			p->bytes_out += n;
-			p->bytes_queued -= n;
 		}
 	}
 
@@ -213,88 +206,14 @@ mproc_dispatch(int fd, short event, void *arg)
 		if (n == 0)
 			break;
 
-		p->msg_in += 1;
 		p->handler(p, &imsg);
 
 		imsg_free(&imsg);
 	}
 
-#if 0
-	if (smtpd_process == PROC_QUEUE)
-		queue_flow_control();
-#endif
-
 	mproc_event_add(p);
 }
 
-/* XXX msgbuf_write() should return n ... */
-static ssize_t
-msgbuf_write2(struct msgbuf *msgbuf)
-{
-	struct iovec	 iov[IOV_MAX];
-	struct ibuf	*buf;
-	unsigned int	 i = 0;
-	ssize_t		 n;
-	struct msghdr	 msg;
-	struct cmsghdr	*cmsg;
-	union {
-		struct cmsghdr	hdr;
-		char		buf[CMSG_SPACE(sizeof(int))];
-	} cmsgbuf;
-
-	memset(&iov, 0, sizeof(iov));
-	memset(&msg, 0, sizeof(msg));
-	TAILQ_FOREACH(buf, &msgbuf->bufs, entry) {
-		if (i >= IOV_MAX)
-			break;
-		iov[i].iov_base = buf->buf + buf->rpos;
-		iov[i].iov_len = buf->wpos - buf->rpos;
-		i++;
-		if (buf->fd != -1)
-			break;
-	}
-
-	msg.msg_iov = iov;
-	msg.msg_iovlen = i;
-
-	if (buf != NULL && buf->fd != -1) {
-		msg.msg_control = (caddr_t)&cmsgbuf.buf;
-		msg.msg_controllen = sizeof(cmsgbuf.buf);
-		cmsg = CMSG_FIRSTHDR(&msg);
-		cmsg->cmsg_len = CMSG_LEN(sizeof(int));
-		cmsg->cmsg_level = SOL_SOCKET;
-		cmsg->cmsg_type = SCM_RIGHTS;
-		*(int *)CMSG_DATA(cmsg) = buf->fd;
-	}
-
-again:
-	if ((n = sendmsg(msgbuf->fd, &msg, 0)) == -1) {
-		if (errno == EINTR)
-			goto again;
-		if (errno == ENOBUFS)
-			errno = EAGAIN;
-		return (-1);
-	}
-
-	if (n == 0) {			/* connection closed */
-		errno = 0;
-		return (0);
-	}
-
-	/*
-	 * assumption: fd got sent if sendmsg sent anything
-	 * this works because fds are passed one at a time
-	 */
-	if (buf != NULL && buf->fd != -1) {
-		close(buf->fd);
-		buf->fd = -1;
-	}
-
-	msgbuf_drain(msgbuf, n);
-
-	return (n);
-}
-
 /* This should go into libutil */
 static ssize_t
 imsg_read_nofd(struct imsgbuf *ibuf)
@@ -306,16 +225,13 @@ imsg_read_nofd(struct imsgbuf *ibuf)
 	buf = ibuf->r.buf + ibuf->r.wpos;
 	len = sizeof(ibuf->r.buf) - ibuf->r.wpos;
 
-    again:
-	if ((n = recv(ibuf->fd, buf, len, 0)) == -1) {
-		if (errno != EINTR && errno != EAGAIN)
-			goto fail;
-		goto again;
+	while ((n = recv(ibuf->fd, buf, len, 0)) == -1) {
+		if (errno != EINTR)
+			return (n);
 	}
 
-        ibuf->r.wpos += n;
-fail:
-        return (n);
+	ibuf->r.wpos += n;
+	return (n);
 }
 
 void
@@ -333,11 +249,6 @@ m_forward(struct mproc *p, struct imsg *imsg)
 		    imsg->hdr.len - sizeof(imsg->hdr),
 		    imsg_to_str(imsg->hdr.type));
 
-	p->msg_out += 1;
-	p->bytes_queued += imsg->hdr.len;
-	if (p->bytes_queued > p->bytes_queued_max)
-		p->bytes_queued_max = p->bytes_queued;
-
 	mproc_event_add(p);
 }
 
@@ -355,11 +266,6 @@ m_compose(struct mproc *p, uint32_t type, uint32_t peerid, pid_t pid, int fd,
 		    len,
 		    imsg_to_str(type));
 
-	p->msg_out += 1;
-	p->bytes_queued += len + IMSG_HEADER_SIZE;
-	if (p->bytes_queued > p->bytes_queued_max)
-		p->bytes_queued_max = p->bytes_queued;
-
 	mproc_event_add(p);
 }
 
@@ -376,11 +282,6 @@ m_composev(struct mproc *p, uint32_t type, uint32_t peerid, pid_t pid,
 	for (i = 0; i < n; i++)
 		len += iov[i].iov_len;
 
-	p->msg_out += 1;
-	p->bytes_queued += IMSG_HEADER_SIZE + len;
-	if (p->bytes_queued > p->bytes_queued_max)
-		p->bytes_queued_max = p->bytes_queued;
-
 	if (type != IMSG_STAT_DECREMENT &&
 	    type != IMSG_STAT_INCREMENT)
 		log_trace(TRACE_MPROC, "mproc: %s -> %s : %zu %s",
@@ -395,17 +296,6 @@ m_composev(struct mproc *p, uint32_t type, uint32_t peerid, pid_t pid,
 void
 m_create(struct mproc *p, uint32_t type, uint32_t peerid, pid_t pid, int fd)
 {
-	if (p->m_buf == NULL) {
-		p->m_alloc = 128;
-		log_trace(TRACE_MPROC, "mproc: %s -> %s: allocating %zu",
-		    proc_name(smtpd_process),
-		    proc_name(p->proc),
-		    p->m_alloc);
-		p->m_buf = malloc(p->m_alloc);
-		if (p->m_buf == NULL)
-			fatal("warn: m_create: malloc");
-	}
-
 	p->m_pos = 0;
 	p->m_type = type;
 	p->m_peerid = peerid;
@@ -424,7 +314,7 @@ m_add(struct mproc *p, const void *data, size_t len)
 		fatal(NULL);
 	}
 
-	alloc = p->m_alloc;
+	alloc = p->m_alloc ? p->m_alloc : 128;
 	while (p->m_pos + len > alloc)
 		alloc *= 2;
 	if (alloc != p->m_alloc) {
@@ -434,7 +324,7 @@ m_add(struct mproc *p, const void *data, size_t len)
 		    p->m_alloc,
 		    alloc);
 
-		tmp = realloc(p->m_buf, alloc);
+		tmp = recallocarray(p->m_buf, p->m_alloc, alloc, 1);
 		if (tmp == NULL)
 			fatal("realloc");
 		p->m_alloc = alloc;
@@ -458,11 +348,6 @@ m_close(struct mproc *p)
 		    p->m_pos,
 		    imsg_to_str(p->m_type));
 
-	p->msg_out += 1;
-	p->bytes_queued += p->m_pos + IMSG_HEADER_SIZE;
-	if (p->bytes_queued > p->bytes_queued_max)
-		p->bytes_queued_max = p->bytes_queued;
-
 	mproc_event_add(p);
 }
 
@@ -479,7 +364,6 @@ m_flush(struct mproc *p)
 	    p->m_pos,
 	    imsg_to_str(p->m_type));
 
-	p->msg_out += 1;
 	p->m_pos = 0;
 
 	imsg_flush(&p->imsgbuf);
@@ -523,149 +407,91 @@ m_is_eom(struct msg *m)
 static inline void
 m_get(struct msg *m, void *dst, size_t sz)
 {
-	if (m->pos + sz > m->end)
-		m_error("msg too short");
+	if (sz > MAX_IMSGSIZE ||
+	    m->end - m->pos < (ssize_t)sz)
+		fatalx("msg too short");
+
 	memmove(dst, m->pos, sz);
 	m->pos += sz;
 }
 
-static inline void
-m_get_typed(struct msg *m, uint8_t type, void *dst, size_t sz)
-{
-	if (m->pos + 1 + sz > m->end)
-		m_error("msg too short");
-	if (*m->pos != type)
-		m_error("msg bad type");
-	memmove(dst, m->pos + 1, sz);
-	m->pos += sz + 1;
-}
-
-static inline void
-m_get_typed_sized(struct msg *m, uint8_t type, const void **dst, size_t *sz)
-{
-	if (m->pos + 1 + sizeof(*sz) > m->end)
-		m_error("msg too short");
-	if (*m->pos != type)
-		m_error("msg bad type");
-	memmove(sz, m->pos + 1, sizeof(*sz));
-	m->pos += sizeof(sz) + 1;
-	if (m->pos + *sz > m->end)
-		m_error("msg too short");
-	*dst = m->pos;
-	m->pos += *sz;
-}
-
-static void
-m_add_typed(struct mproc *p, uint8_t type, const void *data, size_t len)
-{
-	m_add(p, &type, 1);
-	m_add(p, data, len);
-}
-
-static void
-m_add_typed_sized(struct mproc *p, uint8_t type, const void *data, size_t len)
-{
-	m_add(p, &type, 1);
-	m_add(p, &len, sizeof(len));
-	m_add(p, data, len);
-}
-
-enum {
-	M_INT,
-	M_UINT32,
-	M_SIZET,
-	M_TIME,
-	M_STRING,
-	M_DATA,
-	M_ID,
-	M_EVPID,
-	M_MSGID,
-	M_SOCKADDR,
-	M_MAILADDR,
-	M_ENVELOPE,
-};
-
 void
 m_add_int(struct mproc *m, int v)
 {
-	m_add_typed(m, M_INT, &v, sizeof v);
+	m_add(m, &v, sizeof(v));
 };
 
 void
 m_add_u32(struct mproc *m, uint32_t u32)
 {
-	m_add_typed(m, M_UINT32, &u32, sizeof u32);
+	m_add(m, &u32, sizeof(u32));
 };
 
 void
 m_add_size(struct mproc *m, size_t sz)
 {
-	m_add_typed(m, M_SIZET, &sz, sizeof sz);
+	m_add(m, &sz, sizeof(sz));
 };
 
 void
 m_add_time(struct mproc *m, time_t v)
 {
-	m_add_typed(m, M_TIME, &v, sizeof v);
+	m_add(m, &v, sizeof(v));
 };
 
 void
 m_add_string(struct mproc *m, const char *v)
 {
-	m_add_typed(m, M_STRING, v, strlen(v) + 1);
+	m_add(m, v, strlen(v) + 1);
 };
 
 void
 m_add_data(struct mproc *m, const void *v, size_t len)
 {
-	m_add_typed_sized(m, M_DATA, v, len);
+	m_add_size(m, len);
+	m_add(m, v, len);
 };
 
 void
 m_add_id(struct mproc *m, uint64_t v)
 {
-	m_add_typed(m, M_ID, &v, sizeof(v));
+	m_add(m, &v, sizeof(v));
 }
 
 void
 m_add_evpid(struct mproc *m, uint64_t v)
 {
-	m_add_typed(m, M_EVPID, &v, sizeof(v));
+	m_add(m, &v, sizeof(v));
 }
 
 void
 m_add_msgid(struct mproc *m, uint32_t v)
 {
-	m_add_typed(m, M_MSGID, &v, sizeof(v));
+	m_add(m, &v, sizeof(v));
 }
 
 void
 m_add_sockaddr(struct mproc *m, const struct sockaddr *sa)
 {
-	m_add_typed_sized(m, M_SOCKADDR, sa, SA_LEN(sa));
+	m_add_size(m, SA_LEN(sa));
+	m_add(m, sa, SA_LEN(sa));
 }
 
 void
 m_add_mailaddr(struct mproc *m, const struct mailaddr *maddr)
 {
-	m_add_typed(m, M_MAILADDR, maddr, sizeof(*maddr));
+	m_add(m, maddr, sizeof(*maddr));
 }
 
-#ifndef BUILD_FILTER
 void
 m_add_envelope(struct mproc *m, const struct envelope *evp)
 {
-#if 0
-	m_add_typed(m, M_ENVELOPE, evp, sizeof(*evp));
-#else
 	char	buf[sizeof(*evp)];
 
 	envelope_dump_buffer(evp, buf, sizeof(buf));
 	m_add_evpid(m, evp->id);
-	m_add_typed_sized(m, M_ENVELOPE, buf, strlen(buf) + 1);
-#endif
+	m_add_string(m, buf);
 }
-#endif
 
 void
 m_add_params(struct mproc *m, struct dict *d)
@@ -689,25 +515,25 @@ m_add_params(struct mproc *m, struct dict *d)
 void
 m_get_int(struct msg *m, int *i)
 {
-	m_get_typed(m, M_INT, i, sizeof(*i));
+	m_get(m, i, sizeof(*i));
 }
 
 void
 m_get_u32(struct msg *m, uint32_t *u32)
 {
-	m_get_typed(m, M_UINT32, u32, sizeof(*u32));
+	m_get(m, u32, sizeof(*u32));
 }
 
 void
 m_get_size(struct msg *m, size_t *sz)
 {
-	m_get_typed(m, M_SIZET, sz, sizeof(*sz));
+	m_get(m, sz, sizeof(*sz));
 }
 
 void
 m_get_time(struct msg *m, time_t *t)
 {
-	m_get_typed(m, M_TIME, t, sizeof(*t));
+	m_get(m, t, sizeof(*t));
 }
 
 void
@@ -715,79 +541,75 @@ m_get_string(struct msg *m, const char **s)
 {
 	uint8_t	*end;
 
-	if (m->pos + 2 > m->end)
+	if (m->pos >= m->end)
 		m_error("msg too short");
-	if (*m->pos != M_STRING)
-		m_error("bad msg type");
 
-	end = memchr(m->pos + 1, 0, m->end - (m->pos + 1));
+	end = memchr(m->pos, 0, m->end - m->pos);
 	if (end == NULL)
 		m_error("unterminated string");
 
-	*s = m->pos + 1;
+	*s = m->pos;
 	m->pos = end + 1;
 }
 
 void
 m_get_data(struct msg *m, const void **data, size_t *sz)
 {
-	m_get_typed_sized(m, M_DATA, data, sz);
+	m_get_size(m, sz);
+
+	if (m->pos + *sz > m->end)
+		m_error("msg too short");
+
+	*data = m->pos;
+	m->pos += *sz;
 }
 
 void
 m_get_evpid(struct msg *m, uint64_t *evpid)
 {
-	m_get_typed(m, M_EVPID, evpid, sizeof(*evpid));
+	m_get(m, evpid, sizeof(*evpid));
 }
 
 void
 m_get_msgid(struct msg *m, uint32_t *msgid)
 {
-	m_get_typed(m, M_MSGID, msgid, sizeof(*msgid));
+	m_get(m, msgid, sizeof(*msgid));
 }
 
 void
 m_get_id(struct msg *m, uint64_t *id)
 {
-	m_get_typed(m, M_ID, id, sizeof(*id));
+	m_get(m, id, sizeof(*id));
 }
 
 void
 m_get_sockaddr(struct msg *m, struct sockaddr *sa)
 {
-	size_t		 s;
-	const void	*d;
+	size_t len;
 
-	m_get_typed_sized(m, M_SOCKADDR, &d, &s);
-	memmove(sa, d, s);
+	m_get_size(m, &len);
+	m_get(m, sa, len);
 }
 
 void
 m_get_mailaddr(struct msg *m, struct mailaddr *maddr)
 {
-	m_get_typed(m, M_MAILADDR, maddr, sizeof(*maddr));
+	m_get(m, maddr, sizeof(*maddr));
 }
 
-#ifndef BUILD_FILTER
 void
 m_get_envelope(struct msg *m, struct envelope *evp)
 {
-#if 0
-	m_get_typed(m, M_ENVELOPE, evp, sizeof(*evp));
-#else
 	uint64_t	 evpid;
-	size_t		 s;
-	const void	*d;
+	const char	*buf;
 
 	m_get_evpid(m, &evpid);
-	m_get_typed_sized(m, M_ENVELOPE, &d, &s);
+	m_get_string(m, &buf);
 
-	if (!envelope_load_buffer(evp, d, s - 1))
+	if (!envelope_load_buffer(evp, buf, strlen(buf)))
 		fatalx("failed to retrieve envelope");
 	evp->id = evpid;
-#endif
 }
-#endif
 
 void
 m_get_params(struct msg *m, struct dict *d)
