@@ -1,4 +1,4 @@
-/*	$OpenBSD: mta_session.c,v 1.83 2016/05/22 16:31:21 gilles Exp $	*/
+/*	$OpenBSD: mta_session.c,v 1.98 2017/05/24 21:27:32 gilles Exp $	*/
 
 /*
  * Copyright (c) 2008 Pierre-Yves Ritschard <pyr@openbsd.org>
@@ -116,8 +116,8 @@ struct mta_session {
 	int			 use_smtp_tls;
 	int			 ready;
 
-	struct iobuf		 iobuf;
-	struct io		 io;
+	struct event		 ev;
+	struct io		*io;
 	int			 ext;
 
 	size_t			 msgtried;
@@ -137,7 +137,7 @@ struct mta_session {
 
 static void mta_session_init(void);
 static void mta_start(int fd, short ev, void *arg);
-static void mta_io(struct io *, int);
+static void mta_io(struct io *, int, void *);
 static void mta_free(struct mta_session *);
 static void mta_on_ptr(void *, void *, void *);
 static void mta_on_timeout(struct runq *, void *);
@@ -151,6 +151,7 @@ static void mta_response(struct mta_session *, char *);
 static const char * mta_strstate(int);
 static void mta_start_tls(struct mta_session *);
 static int mta_verify_certificate(struct mta_session *);
+static void mta_tls_verified(struct mta_session *);
 static struct mta_session *mta_tree_pop(struct tree *, uint64_t);
 static const char * dsn_strret(enum dsn_ret);
 static const char * dsn_strnotify(uint8_t);
@@ -196,7 +197,6 @@ mta_session(struct mta_relay *relay, struct mta_route *route)
 	s->id = generate_uid();
 	s->relay = relay;
 	s->route = route;
-	s->io.sock = -1;
 
 	if (relay->flags & RELAY_SSL && relay->flags & RELAY_AUTH)
 		s->flags |= MTA_USE_AUTH;
@@ -237,8 +237,8 @@ mta_session(struct mta_relay *relay, struct mta_route *route)
 		 */
 		tv.tv_sec = 0;
 		tv.tv_usec = 0;
-		evtimer_set(&s->io.ev, mta_start, s);
-		evtimer_add(&s->io.ev, &tv);
+		evtimer_set(&s->ev, mta_start, s);
+		evtimer_add(&s->ev, &tv);
 	} else if (waitq_wait(&route->dst->ptrname, mta_on_ptr, s)) {
 		m_create(p_lka,  IMSG_MTA_DNS_PTR, 0, 0, -1);
 		m_add_id(p_lka, s->id);
@@ -281,7 +281,6 @@ mta_session_imsg(struct mproc *p, struct imsg *imsg)
 			mta_flush_task(s, IMSG_MTA_DELIVERY_TEMPFAIL,
 			    "Could not get message fd", 0, 0);
 			mta_enter_state(s, MTA_READY);
-			io_reload(&s->io);
 			return;
 		}
 
@@ -290,7 +289,6 @@ mta_session_imsg(struct mproc *p, struct imsg *imsg)
 			fatal("mta: fdopen");
 
 		mta_enter_state(s, MTA_MAIL);
-		io_reload(&s->io);
 		return;
 
 	case IMSG_MTA_DNS_PTR:
@@ -331,7 +329,7 @@ mta_session_imsg(struct mproc *p, struct imsg *imsg)
 				ssl = ssl_mta_init(NULL, NULL, 0, env->sc_tls_ciphers);
 				if (ssl == NULL)
 					fatal("mta: ssl_mta_init");
-				io_start_tls(&s->io, ssl);
+				io_start_tls(s->io, ssl);
 				return;
 			}
 		}
@@ -343,10 +341,9 @@ mta_session_imsg(struct mproc *p, struct imsg *imsg)
 		    resp_ca_cert->cert, resp_ca_cert->cert_len, env->sc_tls_ciphers);
 		if (ssl == NULL)
 			fatal("mta: ssl_mta_init");
-		io_start_tls(&s->io, ssl);
+		io_start_tls(s->io, ssl);
 
-		explicit_bzero(resp_ca_cert->cert, resp_ca_cert->cert_len);
-		free(resp_ca_cert->cert);
+		freezero(resp_ca_cert->cert, resp_ca_cert->cert_len);
 		free(resp_ca_cert);
 		return;
 
@@ -365,9 +362,8 @@ mta_session_imsg(struct mproc *p, struct imsg *imsg)
 			return;
 		}
 
-		mta_io(&s->io, IO_TLSVERIFIED);
-		io_resume(&s->io, IO_PAUSE_IN);
-		io_reload(&s->io);
+		mta_tls_verified(s);
+		io_resume(s->io, IO_IN);
 		return;
 
 	case IMSG_MTA_LOOKUP_HELO:
@@ -430,8 +426,8 @@ mta_free(struct mta_session *s)
 		runq_cancel(hangon, NULL, s);
 	}
 
-	io_clear(&s->io);
-	iobuf_clear(&s->iobuf);
+	if (s->io)
+		io_free(s->io);
 
 	if (s->task)
 		fatalx("current task should have been deleted already");
@@ -457,7 +453,6 @@ mta_on_timeout(struct runq *runq, void *arg)
 	s->hangon++;
 
 	mta_enter_state(s, MTA_READY);
-	io_reload(&s->io);
 }
 
 static void
@@ -501,8 +496,10 @@ mta_connect(struct mta_session *s)
 			s->helo = xstrdup(env->sc_hostname, "mta_connect");
 	}
 
-	io_clear(&s->io);
-	iobuf_clear(&s->iobuf);
+	if (s->io) {
+		io_free(s->io);
+		s->io = NULL;
+	}
 
 	s->use_smtps = s->use_starttls = s->use_smtp_tls = 0;
 
@@ -560,19 +557,19 @@ mta_connect(struct mta_session *s)
 	    portno, s->route->dst->ptrname);
 
 	mta_enter_state(s, MTA_INIT);
-	iobuf_xinit(&s->iobuf, 0, 0, "mta_connect");
-	io_init(&s->io, -1, s, mta_io, &s->iobuf);
-	io_set_timeout(&s->io, 300000);
-	if (io_connect(&s->io, sa, s->route->src->sa) == -1) {
+	s->io = io_new();
+	io_set_callback(s->io, mta_io, s);
+	io_set_timeout(s->io, 300000);
+	if (io_connect(s->io, sa, s->route->src->sa) == -1) {
 		/*
 		 * This error is most likely a "no route",
 		 * so there is no need to try again.
 		 */
-		log_debug("debug: mta: io_connect failed: %s", s->io.error);
+		log_debug("debug: mta: io_connect failed: %s", io_error(s->io));
 		if (errno == EADDRNOTAVAIL)
-			mta_source_error(s->relay, s->route, s->io.error);
+			mta_source_error(s->relay, s->route, io_error(s->io));
 		else
-			mta_error(s, "Connection failed: %s", s->io.error);
+			mta_error(s, "Connection failed: %s", io_error(s->io));
 		mta_free(s);
 	}
 }
@@ -845,7 +842,7 @@ mta_enter_state(struct mta_session *s, int newstate)
 
 	case MTA_LMTP_EOM:
 		/* LMTP reports status of each delivery, so enable read */
-		io_set_read(&s->io);
+		io_set_read(s->io);
 		break;
 
 	case MTA_RSET:
@@ -1036,7 +1033,7 @@ mta_response(struct mta_session *s, char *line)
 			 */
 			sa_len = sizeof(ss);
 			sa = (struct sockaddr *)&ss;
-			if (getsockname(s->io.sock, sa, &sa_len) < 0)
+			if (getsockname(io_fileno(s->io), sa, &sa_len) < 0)
 				mta_delivery_log(e, NULL, buf, delivery, line);
 			else
 				mta_delivery_log(e, sa_to_text(sa),
@@ -1136,14 +1133,13 @@ mta_response(struct mta_session *s, char *line)
 }
 
 static void
-mta_io(struct io *io, int evt)
+mta_io(struct io *io, int evt, void *arg)
 {
-	struct mta_session	*s = io->arg;
+	struct mta_session	*s = arg;
 	char			*line, *msg, *p;
 	size_t			 len;
 	const char		*error;
 	int			 cont;
-	X509			*x;
 
 	log_trace(TRACE_IO, "mta: %p: %s %s", s, io_strevent(evt),
 	    io_strio(io));
@@ -1165,43 +1161,26 @@ mta_io(struct io *io, int evt)
 
 	case IO_TLSREADY:
 		log_info("%016"PRIx64" mta event=starttls ciphers=%s",
-		    s->id, ssl_to_text(s->io.ssl));
+		    s->id, ssl_to_text(io_ssl(s->io)));
 		s->flags |= MTA_TLS;
 
 		if (mta_verify_certificate(s)) {
-			io_pause(&s->io, IO_PAUSE_IN);
+			io_pause(s->io, IO_IN);
 			break;
 		}
 
-	case IO_TLSVERIFIED:
-		x = SSL_get_peer_certificate(s->io.ssl);
-		if (x) {
-			log_info("smtp-out: Server certificate verification %s "
-			    "on session %016"PRIx64,
-			    (s->flags & MTA_VERIFIED) ? "succeeded" : "failed",
-			    s->id);
-			X509_free(x);
-		}
-
-		if (s->use_smtps) {
-			mta_enter_state(s, MTA_BANNER);
-			io_set_read(io);
-		}
-		else
-			mta_enter_state(s, MTA_EHLO);
+		mta_tls_verified(s);
 		break;
 
 	case IO_DATAIN:
 	    nextline:
-		line = iobuf_getline(&s->iobuf, &len);
+		line = io_getline(s->io, &len);
 		if (line == NULL) {
-			if (iobuf_len(&s->iobuf) >= LINE_MAX) {
+			if (io_datalen(s->io) >= LINE_MAX) {
 				mta_error(s, "Input too long");
 				mta_free(s);
-				return;
 			}
-			iobuf_normalize(&s->iobuf);
-			break;
+			return;
 		}
 
 		log_trace(TRACE_MTA, "mta: %p: <<< %s", s, line);
@@ -1280,9 +1259,7 @@ mta_io(struct io *io, int evt)
 			return;
 		}
 
-		iobuf_normalize(&s->iobuf);
-
-		if (iobuf_len(&s->iobuf)) {
+		if (io_datalen(s->io)) {
 			log_debug("debug: mta: remaining data in input buffer");
 			mta_error(s, "Remote host sent too much data");
 			if (s->flags & MTA_WAIT)
@@ -1301,7 +1278,7 @@ mta_io(struct io *io, int evt)
 			}
 		}
 
-		if (iobuf_queued(&s->iobuf) == 0)
+		if (io_queued(s->io) == 0)
 			io_set_read(io);
 		break;
 
@@ -1315,13 +1292,13 @@ mta_io(struct io *io, int evt)
 		break;
 
 	case IO_ERROR:
-		log_debug("debug: mta: %p: IO error: %s", s, io->error);
+		log_debug("debug: mta: %p: IO error: %s", s, io_error(io));
 		if (!s->ready) {
-			mta_error(s, "IO Error: %s", io->error);
+			mta_error(s, "IO Error: %s", io_error(io));
 			mta_connect(s);
 			break;
 		}
-		else if (!(s->flags & (MTA_FORCE_TLS|MTA_FORCE_ANYSSL))) {
+		else if (!(s->flags & (MTA_FORCE_TLS|MTA_FORCE_SMTPS|MTA_FORCE_ANYSSL))) {
 			/* error in non-strict SSL negotiation, downgrade to plain */
 			if (s->flags & MTA_TLS) {
 				log_info("smtp-out: Error on session %016"PRIx64
@@ -1333,13 +1310,13 @@ mta_io(struct io *io, int evt)
 				break;
 			}
 		}
-		mta_error(s, "IO Error: %s", io->error);
+		mta_error(s, "IO Error: %s", io_error(io));
 		mta_free(s);
 		break;
 
 	case IO_TLSERROR:
-		log_debug("debug: mta: %p: TLS IO error: %s", s, io->error);
-		if (!(s->flags & (MTA_FORCE_TLS|MTA_FORCE_ANYSSL))) {
+		log_debug("debug: mta: %p: TLS IO error: %s", s, io_error(io));
+		if (!(s->flags & (MTA_FORCE_TLS|MTA_FORCE_SMTPS|MTA_FORCE_ANYSSL))) {
 			/* error in non-strict SSL negotiation, downgrade to plain */
 			log_info("smtp-out: TLS Error on session %016"PRIx64
 			    ": TLS failed, "
@@ -1349,7 +1326,7 @@ mta_io(struct io *io, int evt)
 			mta_connect(s);
 			break;
 		}
-		mta_error(s, "IO Error: %s", io->error);
+		mta_error(s, "IO Error: %s", io_error(io));
 		mta_free(s);
 		break;
 
@@ -1382,7 +1359,7 @@ mta_send(struct mta_session *s, char *fmt, ...)
 
 	log_trace(TRACE_MTA, "mta: %p: >>> %s", s, p);
 
-	iobuf_xfqueue(&s->iobuf, "mta_send", "%s\r\n", p);
+	io_xprintf(s->io, "%s\r\n", p);
 
 	free(p);
 }
@@ -1397,15 +1374,14 @@ mta_queue_data(struct mta_session *s)
 	size_t	 sz = 0, q;
 	ssize_t	 len;
 
-	q = iobuf_queued(&s->iobuf);
+	q = io_queued(s->io);
 
-	while (iobuf_queued(&s->iobuf) < MTA_HIWAT) {
+	while (io_queued(s->io) < MTA_HIWAT) {
 		if ((len = getline(&ln, &sz, s->datafp)) == -1)
 			break;
 		if (ln[len - 1] == '\n')
 			ln[len - 1] = '\0';
-		iobuf_xfqueue(&s->iobuf, "mta_queue_data", "%s%s\r\n",
-		    *ln == '.' ? "." : "", ln);
+		io_xprintf(s->io, "%s%s\r\n", *ln == '.' ? "." : "", ln);
 	}
 
 	free(ln);
@@ -1420,7 +1396,7 @@ mta_queue_data(struct mta_session *s)
 		s->datafp = NULL;
 	}
 
-	return (iobuf_queued(&s->iobuf) - q);
+	return (io_queued(s->io) - q);
 }
 
 static void
@@ -1457,7 +1433,7 @@ mta_flush_task(struct mta_session *s, int delivery, const char *error, size_t co
 		 */
 		sa = (struct sockaddr *)&ss;
 		sa_len = sizeof(ss);
-		if (getsockname(s->io.sock, sa, &sa_len) < 0)
+		if (getsockname(io_fileno(s->io), sa, &sa_len) < 0)
 			mta_delivery_log(e, NULL, relay, delivery, error);
 		else
 			mta_delivery_log(e, sa_to_text(sa),
@@ -1584,10 +1560,10 @@ mta_verify_certificate(struct mta_session *s)
 	    >= sizeof req_ca_vrfy.name)
 		return 0;
 
-	x = SSL_get_peer_certificate(s->io.ssl);
+	x = SSL_get_peer_certificate(io_ssl(s->io));
 	if (x == NULL)
 		return 0;
-	xchain = SSL_get_peer_cert_chain(s->io.ssl);
+	xchain = SSL_get_peer_cert_chain(io_ssl(s->io));
 
 	/*
 	 * Client provided a certificate and possibly a certificate chain.
@@ -1676,6 +1652,28 @@ mta_verify_certificate(struct mta_session *s)
 	return res;
 }
 
+static void
+mta_tls_verified(struct mta_session *s)
+{
+	X509 *x;
+
+	x = SSL_get_peer_certificate(io_ssl(s->io));
+	if (x) {
+		log_info("smtp-out: Server certificate verification %s "
+		    "on session %016"PRIx64,
+		    (s->flags & MTA_VERIFIED) ? "succeeded" : "failed",
+		    s->id);
+		X509_free(x);
+	}
+
+	if (s->use_smtps) {
+		mta_enter_state(s, MTA_BANNER);
+		io_set_read(s->io);
+	}
+	else
+		mta_enter_state(s, MTA_EHLO);
+}
+
 static const char *
 dsn_strret(enum dsn_ret ret)
 {
