@@ -1,4 +1,4 @@
-/*	$OpenBSD: smtp.c,v 1.155 2016/03/25 15:06:58 krw Exp $	*/
+/*	$OpenBSD: smtp.c,v 1.157 2017/11/21 12:20:34 eric Exp $	*/
 
 /*
  * Copyright (c) 2008 Gilles Chehade <gilles@poolp.org>
@@ -63,57 +63,46 @@ static size_t	maxsessions;
 void
 smtp_imsg(struct mproc *p, struct imsg *imsg)
 {
-	if (p->proc == PROC_LKA) {
-		switch (imsg->hdr.type) {
-		case IMSG_SMTP_DNS_PTR:
-		case IMSG_SMTP_CHECK_SENDER:
-		case IMSG_SMTP_EXPAND_RCPT:
-		case IMSG_SMTP_LOOKUP_HELO:
-		case IMSG_SMTP_AUTHENTICATE:
-		case IMSG_SMTP_TLS_INIT:
-		case IMSG_SMTP_TLS_VERIFY:
-			smtp_session_imsg(p, imsg);
-			return;
-		}
-	}
+	switch (imsg->hdr.type) {
+	case IMSG_SMTP_DNS_PTR:
+	case IMSG_SMTP_CHECK_SENDER:
+	case IMSG_SMTP_EXPAND_RCPT:
+	case IMSG_SMTP_LOOKUP_HELO:
+	case IMSG_SMTP_AUTHENTICATE:
+	case IMSG_SMTP_TLS_INIT:
+	case IMSG_SMTP_TLS_VERIFY:
+		smtp_session_imsg(p, imsg);
+		return;
 
-	if (p->proc == PROC_QUEUE) {
-		switch (imsg->hdr.type) {
-		case IMSG_SMTP_MESSAGE_COMMIT:
-		case IMSG_SMTP_MESSAGE_CREATE:
-		case IMSG_SMTP_MESSAGE_OPEN:
-		case IMSG_QUEUE_ENVELOPE_SUBMIT:
-		case IMSG_QUEUE_ENVELOPE_COMMIT:
-			smtp_session_imsg(p, imsg);
-			return;
+	case IMSG_SMTP_MESSAGE_COMMIT:
+	case IMSG_SMTP_MESSAGE_CREATE:
+	case IMSG_SMTP_MESSAGE_OPEN:
+	case IMSG_QUEUE_ENVELOPE_SUBMIT:
+	case IMSG_QUEUE_ENVELOPE_COMMIT:
+		smtp_session_imsg(p, imsg);
+		return;
 
-		case IMSG_QUEUE_SMTP_SESSION:
-			m_compose(p, IMSG_QUEUE_SMTP_SESSION, 0, 0,
-			    smtp_enqueue(), imsg->data,
-			    imsg->hdr.len - sizeof imsg->hdr);
-			return;
-		}
-	}
+	case IMSG_QUEUE_SMTP_SESSION:
+		m_compose(p, IMSG_QUEUE_SMTP_SESSION, 0, 0, smtp_enqueue(),
+		    imsg->data, imsg->hdr.len - sizeof imsg->hdr);
+		return;
 
-	if (p->proc == PROC_CONTROL) {
-		switch (imsg->hdr.type) {
-		case IMSG_CTL_SMTP_SESSION:
-			m_compose(p, IMSG_CTL_SMTP_SESSION, imsg->hdr.peerid, 0,
-			    smtp_enqueue(), NULL, 0);
-			return;
+	case IMSG_CTL_SMTP_SESSION:
+		m_compose(p, IMSG_CTL_SMTP_SESSION, imsg->hdr.peerid, 0,
+		    smtp_enqueue(), NULL, 0);
+		return;
 
-		case IMSG_CTL_PAUSE_SMTP:
-			log_debug("debug: smtp: pausing listening sockets");
-			smtp_pause();
-			env->sc_flags |= SMTPD_SMTP_PAUSED;
-			return;
+	case IMSG_CTL_PAUSE_SMTP:
+		log_debug("debug: smtp: pausing listening sockets");
+		smtp_pause();
+		env->sc_flags |= SMTPD_SMTP_PAUSED;
+		return;
 
-		case IMSG_CTL_RESUME_SMTP:
-			log_debug("debug: smtp: resuming listening sockets");
-			env->sc_flags &= ~SMTPD_SMTP_PAUSED;
-			smtp_resume();
-			return;
-		}
+	case IMSG_CTL_RESUME_SMTP:
+		log_debug("debug: smtp: resuming listening sockets");
+		env->sc_flags &= ~SMTPD_SMTP_PAUSED;
+		smtp_resume();
+		return;
 	}
 
 	errx(1, "smtp_imsg: unexpected %s imsg", imsg_to_str(imsg->hdr.type));
@@ -274,6 +263,7 @@ smtp_accept(int fd, short event, void *p)
 	struct sockaddr_storage	 ss;
 	socklen_t		 len;
 	int			 sock;
+	int			 ret;
 
 	if (env->sc_flags & SMTPD_SMTP_PAUSED)
 		fatalx("smtp_session: unexpected client");
@@ -296,7 +286,12 @@ smtp_accept(int fd, short event, void *p)
 		fatal("smtp_accept");
 	}
 
-	if (smtp_session(listener, sock, &ss, NULL) == -1) {
+	if (listener->filter[0])
+		ret = smtpf_session(listener, sock, &ss, NULL);
+	else
+		ret = smtp_session(listener, sock, &ss, NULL);
+
+	if (ret == -1) {
 		log_warn("warn: Failed to create SMTP session");
 		close(sock);
 		return;
