@@ -1,4 +1,4 @@
-/*	$OpenBSD: smtpctl.c,v 1.149 2016/04/29 08:55:08 eric Exp $	*/
+/*	$OpenBSD: smtpctl.c,v 1.154 2017/07/27 18:48:30 sunil Exp $	*/
 
 /*
  * Copyright (c) 2013 Eric Faurot <eric@openbsd.org>
@@ -65,7 +65,7 @@
 #endif
 
 #ifndef HAVE_DB_API
-#define	makemap(x, y)	1
+#define	makemap(x, y, z)	1
 #endif
 
 
@@ -237,45 +237,51 @@ srv_read(void *dst, size_t sz)
 static void
 srv_get_int(int *i)
 {
-	uint8_t type;
-
-	srv_read(&type, 1);
 	srv_read(i, sizeof(*i));
 }
 
 static void
 srv_get_time(time_t *t)
 {
-	uint8_t type;
-
-	srv_read(&type, 1);
 	srv_read(t, sizeof(*t));
 }
 
 static void
 srv_get_evpid(uint64_t *evpid)
 {
-	uint8_t type;
-
-	srv_read(&type, 1);
 	srv_read(evpid, sizeof(*evpid));
 }
 
+static void
+srv_get_string(const char **s)
+{
+	const char *end;
+	size_t len;
+
+	if (rlen == 0)
+		errx(1, "message too short");
+
+	end = memchr(rdata, 0, rlen);
+	if (end == NULL)
+		errx(1, "unterminated string");
+
+	len = end + 1 - rdata;
+
+	*s = rdata;
+	rlen -= len;
+	rdata += len;
+}
+
 static void
 srv_get_envelope(struct envelope *evp)
 {
 	uint64_t	 evpid;
-	uint8_t		 type;
-	size_t		 s;
-	const void	*d;
+	const char	*str;
 
 	srv_get_evpid(&evpid);
-	srv_read(&type, sizeof(type));
-	srv_read(&s, sizeof(s));
-	d = rdata;
-	srv_read(NULL, s);
+	srv_get_string(&str);
 
-	envelope_load_buffer(evp, d, s - 1);
+	envelope_load_buffer(evp, str, strlen(str));
 	evp->id = evpid;
 }
 
@@ -407,26 +413,20 @@ static int
 srv_iter_evpids(uint32_t msgid, uint64_t *evpid, int *offset)
 {
 	static uint64_t	*evpids = NULL, *tmp;
-	static int	 n, alloc = 0;
+	static int	 n, tmpalloc, alloc = 0;
 	struct envelope	 evp;
 
-	if (evpids == NULL) {
-		alloc = 1000;
-		evpids = calloc(alloc, sizeof(*evpids));
-		if (evpids == NULL)
-			err(1, "calloc");
-	}
-
 	if (*offset == 0) {
 		n = 0;
 		while (srv_iter_envelopes(msgid, &evp)) {
 			if (n == alloc) {
-				alloc += 256;
-				tmp = reallocarray(evpids, alloc,
+				tmpalloc = alloc ? (alloc * 2) : 128;
+				tmp = recallocarray(evpids, alloc, tmpalloc,
 				    sizeof(*evpids));
 				if (tmp == NULL)
-					err(1, "reallocarray");
+					err(1, "recallocarray");
 				evpids = tmp;
+				alloc = tmpalloc;
 			}
 			evpids[n++] = evp.id;
 		}
@@ -741,7 +741,7 @@ do_show_queue(int argc, struct parameter *argv)
 	now = time(NULL);
 
 	if (!srv_connect()) {
-		log_init(1);
+		log_init(1, LOG_MAIL);
 		queue_init("fs", 0);
 		if (chroot(PATH_SPOOL) == -1 || chdir("/") == -1)
 			err(1, "%s", PATH_SPOOL);
@@ -874,13 +874,6 @@ do_show_status(int argc, struct parameter *argv)
 	return (0);
 }
 
-static int
-do_stop(int argc, struct parameter *argv)
-{
-	srv_send(IMSG_CTL_SHUTDOWN, NULL, 0);
-	return srv_check_result(1);
-}
-
 static int
 do_trace(int argc, struct parameter *argv)
 {
@@ -1042,6 +1035,8 @@ main(int argc, char **argv)
 	gid_t		 gid;
 	char		*argv_mailq[] = { "show", "queue", NULL };
 
+	__progname = ssh_get_progname(argv[0]);
+
 	sendmail_compat(argc, argv);
 	if (geteuid())
 		errx(1, "need root privileges");
@@ -1091,7 +1086,6 @@ main(int argc, char **argv)
 	cmd_install("show routes",		do_show_routes);
 	cmd_install("show stats",		do_show_stats);
 	cmd_install("show status",		do_show_status);
-	cmd_install("stop",			do_stop);
 	cmd_install("trace <str>",		do_trace);
 	cmd_install("uncorrupt <msgid>",	do_uncorrupt);
 	cmd_install("unprofile <str>",		do_unprofile);
@@ -1121,10 +1115,8 @@ sendmail_compat(int argc, char **argv)
 		 * that should invoke makemap/newaliases.
 		 */
 		for (i = 1; i < argc; i++)
-			if (strncmp(argv[i], "-bi", 3) == 0) {
-				__progname = "newaliases";
-				exit(makemap(argc, argv));
-			}
+			if (strncmp(argv[i], "-bi", 3) == 0)
+				exit(makemap(P_NEWALIASES, argc, argv));
 
 		if (!srv_connect())
 			offlinefp = offline_file();
@@ -1140,9 +1132,10 @@ sendmail_compat(int argc, char **argv)
 
 		sendmail = 1;
 		exit(enqueue(argc, argv, offlinefp));
-	} else if (strcmp(__progname, "makemap") == 0 ||
-	    strcmp(__progname, "newaliases") == 0)
-		exit(makemap(argc, argv));
+	} else if (strcmp(__progname, "makemap") == 0)
+		exit(makemap(P_MAKEMAP, argc, argv));
+	else if (strcmp(__progname, "newaliases") == 0)
+		exit(makemap(P_NEWALIASES, argc, argv));
 }
 
 static void
