@@ -1,4 +1,4 @@
-/*	$OpenBSD: smtpd-defines.h,v 1.6 2015/11/30 14:39:21 gilles Exp $	*/
+/*	$OpenBSD: smtpd-defines.h,v 1.7 2016/08/31 10:18:08 gilles Exp $	*/
 
 /*
  * Copyright (c) 2013 Gilles Chehade <gilles@poolp.org>
@@ -47,7 +47,7 @@
 #define PATH_SPOOL		"/var/spool/smtpd"
 #endif
 
-#define TAG_CHAR		'+'
+#define SUBADDRESSING_DELIMITER	"+"
 
 
 /* sendmail compat */
