@@ -1,4 +1,4 @@
-/*	$OpenBSD: smtpd.c,v 1.279 2016/06/20 20:26:04 gilles Exp $	*/
+/*	$OpenBSD: smtpd.c,v 1.291 2017/11/21 12:20:34 eric Exp $	*/
 
 /*
  * Copyright (c) 2008 Gilles Chehade <gilles@poolp.org>
@@ -91,7 +91,7 @@ extern char *__progname;
 static void parent_imsg(struct mproc *, struct imsg *);
 static void usage(void);
 static int smtpd(void);
-static void parent_shutdown(int);
+static void parent_shutdown(void);
 static void parent_send_config(int, short, void *);
 static void parent_send_config_lka(void);
 static void parent_send_config_pony(void);
@@ -170,10 +170,8 @@ const char	*backend_scheduler = "ramqueue";
 const char	*backend_stat = "ram";
 
 int	profiling = 0;
-int	verbose = 0;
 int	debug = 0;
 int	foreground = 0;
-int	foreground_log = 0;
 int	control_socket = -1;
 
 struct tree	 children;
@@ -197,106 +195,92 @@ parent_imsg(struct mproc *p, struct imsg *imsg)
 	int			 fd, n, v, ret;
 
 	if (imsg == NULL)
-		exit(1);
-	
-	if (p->proc == PROC_LKA) {
-		switch (imsg->hdr.type) {
-		case IMSG_LKA_OPEN_FORWARD:
-			CHECK_IMSG_DATA_SIZE(imsg, sizeof *fwreq);
-			fwreq = imsg->data;
-			fd = parent_forward_open(fwreq->user, fwreq->directory,
-			    fwreq->uid, fwreq->gid);
-			fwreq->status = 0;
-			if (fd == -1 && errno != ENOENT) {
-				if (errno == EAGAIN)
-					fwreq->status = -1;
-			}
-			else
-				fwreq->status = 1;
-			m_compose(p, IMSG_LKA_OPEN_FORWARD, 0, 0, fd,
-			    fwreq, sizeof *fwreq);
-			return;
+		fatalx("process %s socket closed", p->name);
 
-		case IMSG_LKA_AUTHENTICATE:
-			/*
-			 * If we reached here, it means we want root to lookup
-			 * system user.
-			 */
-			m_msg(&m, imsg);
-			m_get_id(&m, &reqid);
-			m_get_string(&m, &username);
-			m_get_string(&m, &password);
-			m_end(&m);
+	switch (imsg->hdr.type) {
+	case IMSG_LKA_OPEN_FORWARD:
+		CHECK_IMSG_DATA_SIZE(imsg, sizeof *fwreq);
+		fwreq = imsg->data;
+		fd = parent_forward_open(fwreq->user, fwreq->directory,
+		    fwreq->uid, fwreq->gid);
+		fwreq->status = 0;
+		if (fd == -1 && errno != ENOENT) {
+			if (errno == EAGAIN)
+				fwreq->status = -1;
+		}
+		else
+			fwreq->status = 1;
+		m_compose(p, IMSG_LKA_OPEN_FORWARD, 0, 0, fd,
+		    fwreq, sizeof *fwreq);
+		return;
 
-			ret = parent_auth_user(username, password);
+	case IMSG_LKA_AUTHENTICATE:
+		/*
+		 * If we reached here, it means we want root to lookup
+		 * system user.
+		 */
+		m_msg(&m, imsg);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &username);
+		m_get_string(&m, &password);
+		m_end(&m);
 
-			m_create(p, IMSG_LKA_AUTHENTICATE, 0, 0, -1);
-			m_add_id(p, reqid);
-			m_add_int(p, ret);
-			m_close(p);
+		ret = parent_auth_user(username, password);
+
+		m_create(p, IMSG_LKA_AUTHENTICATE, 0, 0, -1);
+		m_add_id(p, reqid);
+		m_add_int(p, ret);
+		m_close(p);
+		return;
+
+	case IMSG_MDA_FORK:
+		m_msg(&m, imsg);
+		m_get_id(&m, &reqid);
+		m_get_data(&m, &data, &sz);
+		m_end(&m);
+		if (sz != sizeof(deliver))
+			fatalx("expected deliver");
+		memmove(&deliver, data, sz);
+		forkmda(p, reqid, &deliver);
+		return;
+
+	case IMSG_MDA_KILL:
+		m_msg(&m, imsg);
+		m_get_id(&m, &reqid);
+		m_get_string(&m, &cause);
+		m_end(&m);
+
+		i = NULL;
+		while ((n = tree_iter(&children, &i, NULL, (void**)&c)))
+			if (c->type == CHILD_MDA &&
+			    c->mda_id == reqid &&
+			    c->cause == NULL)
+				break;
+		if (!n) {
+			log_debug("debug: smtpd: "
+			    "kill request: proc not found");
 			return;
 		}
-	}
 
-	if (p->proc == PROC_PONY) {
-		switch (imsg->hdr.type) {
-		case IMSG_MDA_FORK:
-			m_msg(&m, imsg);
-			m_get_id(&m, &reqid);
-			m_get_data(&m, &data, &sz);
-			m_end(&m);
-			if (sz != sizeof(deliver))
-				fatalx("expected deliver");
-			memmove(&deliver, data, sz);
-			forkmda(p, reqid, &deliver);
-			return;
+		c->cause = xstrdup(cause, "parent_imsg");
+		log_debug("debug: smtpd: kill requested for %u: %s",
+		    c->pid, c->cause);
+		kill(c->pid, SIGTERM);
+		return;
 
-		case IMSG_MDA_KILL:
-			m_msg(&m, imsg);
-			m_get_id(&m, &reqid);
-			m_get_string(&m, &cause);
-			m_end(&m);
+	case IMSG_CTL_VERBOSE:
+		m_msg(&m, imsg);
+		m_get_int(&m, &v);
+		m_end(&m);
+		log_trace_verbose(v);
+		return;
 
-			i = NULL;
-			while ((n = tree_iter(&children, &i, NULL, (void**)&c)))
-				if (c->type == CHILD_MDA &&
-				    c->mda_id == reqid &&
-				    c->cause == NULL)
-					break;
-			if (!n) {
-				log_debug("debug: smtpd: "
-				    "kill request: proc not found");
-				return;
-			}
-
-			c->cause = xstrdup(cause, "parent_imsg");
-			log_debug("debug: smtpd: kill requested for %u: %s",
-			    c->pid, c->cause);
-			kill(c->pid, SIGTERM);
-			return;
-		}
-	}
-
-	if (p->proc == PROC_CONTROL) {
-		switch (imsg->hdr.type) {
-		case IMSG_CTL_VERBOSE:
-			m_msg(&m, imsg);
-			m_get_int(&m, &v);
-			m_end(&m);
-			log_verbose(v);
-			return;
-
-		case IMSG_CTL_PROFILE:
-			m_msg(&m, imsg);
-			m_get_int(&m, &v);
-			m_end(&m);
-			profiling = v;
-			return;
-
-		case IMSG_CTL_SHUTDOWN:
-			parent_shutdown(0);
-			return;
-		}
+	case IMSG_CTL_PROFILE:
+		m_msg(&m, imsg);
+		m_get_int(&m, &v);
+		m_end(&m);
+		profiling = v;
+		return;
 	}
 
 	errx(1, "parent_imsg: unexpected %s imsg from %s",
@@ -308,22 +292,22 @@ usage(void)
 {
 	extern char	*__progname;
 
-	fprintf(stderr, "usage: %s [-dhnv] [-D macro=value] "
+	fprintf(stderr, "usage: %s [-dFhnv] [-D macro=value] "
 	    "[-f file] [-P system] [-T trace]\n", __progname);
 	exit(1);
 }
 
 static void
-parent_shutdown(int ret)
+parent_shutdown(void)
 {
-	void		*iter;
-	struct child	*child;
-	pid_t		 pid;
+	pid_t pid;
 
-	iter = NULL;
-	while (tree_iter(&children, &iter, NULL, (void**)&child))
-		if (child->type == CHILD_DAEMON)
-			kill(child->pid, SIGTERM);
+	mproc_clear(p_ca);
+	mproc_clear(p_pony);
+	mproc_clear(p_control);
+	mproc_clear(p_lka);
+	mproc_clear(p_scheduler);
+	mproc_clear(p_queue);
 
 	do {
 		pid = waitpid(WAIT_MYPGRP, NULL, 0);
@@ -331,8 +315,8 @@ parent_shutdown(int ret)
 
 	unlink(SMTPD_SOCKET);
 
-	log_warnx("warn: parent terminating");
-	exit(ret);
+	log_info("Exiting");
+	exit(0);
 }
 
 static void
@@ -372,16 +356,17 @@ static void
 parent_sig_handler(int sig, short event, void *p)
 {
 	struct child	*child;
-	int		 die = 0, die_gracefully = 0, status, fail;
+	int		 status, fail;
 	pid_t		 pid;
 	char		*cause;
 
 	switch (sig) {
 	case SIGTERM:
 	case SIGINT:
-		log_info("info: %s, shutting down", strsignal(sig));
-		die_gracefully = 1;
-		/* FALLTHROUGH */
+		log_debug("debug: got signal %d", sig);
+		parent_shutdown();
+		/* NOT REACHED */
+
 	case SIGCHLD:
 		do {
 			int len;
@@ -418,7 +403,6 @@ parent_sig_handler(int sig, short event, void *p)
 
 			switch (child->type) {
 			case CHILD_DAEMON:
-				die = 1;
 				if (fail)
 					log_warnx("warn: lost child: %s %s",
 					    child->title, cause);
@@ -473,10 +457,6 @@ parent_sig_handler(int sig, short event, void *p)
 			free(cause);
 		} while (pid > 0 || (pid == -1 && errno == EINTR));
 
-		if (die)
-			parent_shutdown(1);
-		else if (die_gracefully)
-			parent_shutdown(0);
 		break;
 	default:
 		fatalx("smtpd: unexpected signal");
@@ -518,9 +498,9 @@ main(int argc, char *argv[])
 	flags = 0;
 	opts = 0;
 	debug = 0;
-	verbose = 0;
+	tracing = 0;
 
-	log_init(1);
+	log_init(1, LOG_MAIL);
 
 	TAILQ_INIT(&offline_q);
 
@@ -565,49 +545,47 @@ main(int argc, char *argv[])
 
 		case 'T':
 			if (!strcmp(optarg, "imsg"))
-				verbose |= TRACE_IMSG;
+				tracing |= TRACE_IMSG;
 			else if (!strcmp(optarg, "io"))
-				verbose |= TRACE_IO;
+				tracing |= TRACE_IO;
 			else if (!strcmp(optarg, "smtp"))
-				verbose |= TRACE_SMTP;
+				tracing |= TRACE_SMTP;
 			else if (!strcmp(optarg, "mfa") ||
 			    !strcmp(optarg, "filter") ||
 			    !strcmp(optarg, "filters"))
-				verbose |= TRACE_FILTERS;
+				tracing |= TRACE_FILTERS;
 			else if (!strcmp(optarg, "mta") ||
 			    !strcmp(optarg, "transfer"))
-				verbose |= TRACE_MTA;
+				tracing |= TRACE_MTA;
 			else if (!strcmp(optarg, "bounce") ||
 			    !strcmp(optarg, "bounces"))
-				verbose |= TRACE_BOUNCE;
+				tracing |= TRACE_BOUNCE;
 			else if (!strcmp(optarg, "scheduler"))
-				verbose |= TRACE_SCHEDULER;
+				tracing |= TRACE_SCHEDULER;
 			else if (!strcmp(optarg, "lookup"))
-				verbose |= TRACE_LOOKUP;
+				tracing |= TRACE_LOOKUP;
 			else if (!strcmp(optarg, "stat") ||
 			    !strcmp(optarg, "stats"))
-				verbose |= TRACE_STAT;
+				tracing |= TRACE_STAT;
 			else if (!strcmp(optarg, "rules"))
-				verbose |= TRACE_RULES;
+				tracing |= TRACE_RULES;
 			else if (!strcmp(optarg, "mproc"))
-				verbose |= TRACE_MPROC;
+				tracing |= TRACE_MPROC;
 			else if (!strcmp(optarg, "expand"))
-				verbose |= TRACE_EXPAND;
+				tracing |= TRACE_EXPAND;
 			else if (!strcmp(optarg, "table") ||
 			    !strcmp(optarg, "tables"))
-				verbose |= TRACE_TABLES;
+				tracing |= TRACE_TABLES;
 			else if (!strcmp(optarg, "queue"))
-				verbose |= TRACE_QUEUE;
+				tracing |= TRACE_QUEUE;
 			else if (!strcmp(optarg, "all"))
-				verbose |= ~TRACE_DEBUG;
+				tracing |= ~TRACE_DEBUG;
 			else if (!strcmp(optarg, "profstat"))
 				profiling |= PROFILE_TOSTAT;
 			else if (!strcmp(optarg, "profile-imsg"))
 				profiling |= PROFILE_IMSG;
 			else if (!strcmp(optarg, "profile-queue"))
 				profiling |= PROFILE_QUEUE;
-			else if (!strcmp(optarg, "profile-buffers"))
-				profiling |= PROFILE_BUFFERS;
 			else
 				log_warnx("warn: unknown trace flag \"%s\"",
 				    optarg);
@@ -621,7 +599,7 @@ main(int argc, char *argv[])
 				flags |= SMTPD_MDA_PAUSED;
 			break;
 		case 'v':
-			verbose |=  TRACE_DEBUG;
+			tracing |=  TRACE_DEBUG;
 			break;
 		case 'x':
 			rexec = optarg;
@@ -669,9 +647,8 @@ main(int argc, char *argv[])
 	if (geteuid())
 		errx(1, "need root privileges");
 
-	log_init(foreground_log);
-	log_verbose(verbose);
-
+	log_init(foreground_log, LOG_MAIL);
+	log_trace_verbose(tracing);
 	load_pki_tree();
 	load_pki_keys();
 
@@ -767,7 +744,7 @@ main(int argc, char *argv[])
 		setup_done(p_queue);
 		setup_done(p_scheduler);
 
-		log_info("smtpd: setup done");
+		log_debug("smtpd: setup done");
 
 		return smtpd();
 	}
@@ -874,11 +851,11 @@ start_child(int save_argc, char **save_argv, char *rexec)
 		return p;
 	}
 
-	close(sp[1]);
-
 	if (dup2(sp[0], 3) == -1)
 		fatal("%s: dup2", rexec);
 
+	xclosefrom(4);
+
 	for (argc = 0; argc < save_argc; argc++)
 		argv[argc] = save_argv[argc];
 	argv[argc++] = "-x";
@@ -929,7 +906,7 @@ setup_done(struct mproc *p)
 	if (imsg.hdr.type != IMSG_SETUP_DONE)
 		fatalx("expect IMSG_SETUP_DONE");
 
-	log_info("setup_done: %s[%d] done", p->name, p->pid);
+	log_debug("setup_done: %s[%d] done", p->name, p->pid);
 
 	imsg_free(&imsg);
 }
@@ -941,6 +918,8 @@ setup_proc(void)
 	struct imsg imsg;
         int setup = 1;
 
+	log_procinit(proc_title(smtpd_process));
+
 	p_parent = calloc(1, sizeof(*p_parent));
 	if (p_parent == NULL)
 		fatal("calloc");
@@ -978,7 +957,7 @@ setup_proc(void)
 	if (imsg_flush(ibuf) == -1)
 		fatal("imsg_flush");
 
-	log_info("setup_proc: %s done", proc_title(smtpd_process));
+	log_debug("setup_proc: %s done", proc_title(smtpd_process));
 }
 
 static struct mproc *
@@ -986,7 +965,7 @@ setup_peer(enum smtp_proc_type proc, pid_t pid, int sock)
 {
 	struct mproc *p, **pp;
 
-	log_info("setup_peer: %s -> %s[%u] fd=%d", proc_title(smtpd_process),
+	log_debug("setup_peer: %s -> %s[%u] fd=%d", proc_title(smtpd_process),
 	    proc_title(proc), pid, sock);
 
 	if (sock == -1)
@@ -1097,7 +1076,6 @@ smtpd(void) {
 	config_peer(PROC_QUEUE);
 	config_peer(PROC_CA);
 	config_peer(PROC_PONY);
-	config_done();
 
 	evtimer_set(&config_ev, parent_send_config, NULL);
 	memset(&tv, 0, sizeof(tv));
@@ -1118,8 +1096,8 @@ smtpd(void) {
 	    "getpw sendfd proc exec id inet unix", NULL) == -1)
 		err(1, "pledge");
 
-	if (event_dispatch() < 0)
-		fatal("smtpd: event_dispatch");
+	event_dispatch();
+	fatalx("exited event loop");
 
 	return (0);
 }
@@ -1265,8 +1243,8 @@ purge_task(void)
 			log_warn("warn: purge_task: fork");
 			break;
 		case 0:
-			if ((pw = getpwnam(SMTPD_USER)) == NULL)
-				fatalx("unknown user " SMTPD_USER);
+			if ((pw = getpwnam(SMTPD_QUEUE_USER)) == NULL)
+				fatalx("unknown user " SMTPD_QUEUE_USER);
 			if (chroot(PATH_SPOOL PATH_PURGE) == -1)
 				fatal("smtpd: chroot");
 			if (chdir("/") == -1)
@@ -1525,8 +1503,6 @@ offline_enqueue(char *name)
 		    setresuid(pw->pw_uid, pw->pw_uid, pw->pw_uid))
 			_exit(1);
 
-		closefrom(STDERR_FILENO + 1);
-
 		if ((fp = fdopen(fd, "r")) == NULL)
 			_exit(1);
 
@@ -1663,7 +1639,7 @@ imsg_dispatch(struct mproc *p, struct imsg *imsg)
 	int		msg;
 
 	if (imsg == NULL) {
-		exit(1);
+		imsg_callback(p, imsg);
 		return;
 	}
 
@@ -1767,8 +1743,6 @@ proc_name(enum smtp_proc_type proc)
 		return "pony";
 	case PROC_CA:
 		return "ca";
-	case PROC_FILTER:
-		return "filter-proc";
 	case PROC_CLIENT:
 		return "client-proc";
 	default:
@@ -1815,7 +1789,6 @@ imsg_to_str(int type)
 	CASE(IMSG_CTL_REMOVE);
 	CASE(IMSG_CTL_SCHEDULE);
 	CASE(IMSG_CTL_SHOW_STATUS);
-	CASE(IMSG_CTL_SHUTDOWN);
 	CASE(IMSG_CTL_TRACE_DISABLE);
 	CASE(IMSG_CTL_TRACE_ENABLE);
 	CASE(IMSG_CTL_UPDATE_TABLE);
