@@ -1,4 +1,4 @@
-/*	$OpenBSD: ssl.c,v 1.86 2016/04/21 14:27:41 jsing Exp $	*/
+/*	$OpenBSD: ssl.c,v 1.89 2017/05/17 14:00:06 deraadt Exp $	*/
 
 /*
  * Copyright (c) 2008 Pierre-Yves Ritschard <pyr@openbsd.org>
@@ -242,10 +242,8 @@ ssl_load_key(const char *name, off_t *len, char *pass, mode_t perm, const char *
 fail:
 	ssl_error("ssl_load_key");
 	free(buf);
-	if (bio != NULL)
-		BIO_free_all(bio);
-	if (key != NULL)
-		EVP_PKEY_free(key);
+	BIO_free_all(bio);
+	EVP_PKEY_free(key);
 	if (fp)
 		fclose(fp);
 	return (NULL);
@@ -269,6 +267,8 @@ ssl_ctx_create(const char *pkiname, char *cert, off_t cert_len, const char *ciph
 	    SSL_OP_ALL | SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_TICKET);
 	SSL_CTX_set_options(ctx,
 	    SSL_OP_NO_SESSION_RESUMPTION_ON_RENEGOTIATION);
+	SSL_CTX_set_options(ctx, SSL_OP_NO_CLIENT_RENEGOTIATION);
+	SSL_CTX_set_options(ctx, SSL_OP_CIPHER_SERVER_PREFERENCE);
 
 	if (ciphers == NULL)
 		ciphers = SSL_CIPHERS;
@@ -397,14 +397,10 @@ ssl_load_pkey(const void *data, size_t datalen, char *buf, off_t len,
 	return (1);
 
  fail:
-	if (rsa != NULL)
-		RSA_free(rsa);
-	if (in != NULL)
-		BIO_free(in);
-	if (pkey != NULL)
-		EVP_PKEY_free(pkey);
-	if (x509 != NULL)
-		X509_free(x509);
+	RSA_free(rsa);
+	BIO_free(in);
+	EVP_PKEY_free(pkey);
+	X509_free(x509);
 	free(exdata);
 
 	return (0);
@@ -433,12 +429,12 @@ ssl_ctx_fake_private_key(SSL_CTX *ctx, const void *data, size_t datalen,
 
 	if (pkeyptr != NULL)
 		*pkeyptr = pkey;
-	else if (pkey != NULL)
+	else
 		EVP_PKEY_free(pkey);
 
 	if (x509ptr != NULL)
 		*x509ptr = x509;
-	else if (x509 != NULL)
+	else
 		X509_free(x509);
 
 	return (ret);
