@@ -1,4 +1,4 @@
-/*	$OpenBSD: table.c,v 1.23 2016/01/04 13:30:20 jung Exp $	*/
+/*	$OpenBSD: table.c,v 1.24 2017/05/01 09:29:07 gilles Exp $	*/
 
 /*
  * Copyright (c) 2013 Eric Faurot <eric@openbsd.org>
@@ -637,8 +637,9 @@ table_parse_lookup(enum table_service service, const char *key,
 static const char *
 table_dump_lookup(enum table_service s, union lookup *lk)
 {
-	static char	buf[LINE_MAX];
-	int		ret;
+	static char		buf[LINE_MAX];
+	struct maddrnode       *mn;
+	int			ret;
 
 	switch (s) {
 	case K_NONE:
@@ -694,6 +695,23 @@ table_dump_lookup(enum table_service s, union lookup *lk)
 			goto err;
 		break;
 
+	case K_MAILADDRMAP:
+		buf[0] = '\0';
+		TAILQ_FOREACH(mn, &lk->maddrmap->queue, entries) {
+			(void)strlcat(buf, mn->mailaddr.user, sizeof(buf));
+			(void)strlcat(buf, "@", sizeof(buf));
+			ret = strlcat(buf, mn->mailaddr.domain, sizeof(buf));
+
+			if (mn != TAILQ_LAST(&lk->maddrmap->queue, xmaddr))
+				ret = strlcat(buf, ", ", sizeof(buf));
+
+			if (ret >= sizeof(buf)) {
+				strlcpy(buf + sizeof(buf) - 4, "...", 4);
+				break;
+			}
+		}
+		break;
+
 	case K_ADDRNAME:
 		ret = snprintf(buf, sizeof(buf), "%s",
 		    lk->addrname.name);
@@ -702,6 +720,7 @@ table_dump_lookup(enum table_service s, union lookup *lk)
 		break;
 
 	default:
+		(void)strlcpy(buf, "???", sizeof(buf));
 		break;
 	}
 
