@@ -200,7 +200,7 @@ time_to_text(time_t when)
 	char *day[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
 	char *month[] = {"Jan","Feb","Mar","Apr","May","Jun",
 			 "Jul","Aug","Sep","Oct","Nov","Dec"};
-	char *tz;
+	const char *tz;
 	long offset;
 
 	lt = localtime(&when);
