@@ -1,4 +1,4 @@
-/*	$OpenBSD: util.c,v 1.127 2016/05/16 17:43:18 gilles Exp $	*/
+/*	$OpenBSD: util.c,v 1.132 2017/01/09 14:49:22 reyk Exp $	*/
 
 /*
  * Copyright (c) 2000,2001 Markus Friedl.  All rights reserved.
@@ -57,6 +57,9 @@ const char *log_in6addr(const struct in6_addr *);
 const char *log_sockaddr(struct sockaddr *);
 static int  parse_mailname_file(char *, size_t);
 
+int	tracing = 0;
+int	foreground_log = 0;
+
 void *
 xmalloc(size_t size, const char *where)
 {
@@ -111,29 +114,31 @@ xmemdup(const void *ptr, size_t size, const char *where)
 }
 
 #if !defined(NO_IO)
-void
-iobuf_xinit(struct iobuf *io, size_t size, size_t max, const char *where)
-{
-	if (iobuf_init(io, size, max) == -1) {
-		log_warnx("%s: iobuf_init(%p, %zu, %zu)", where, io, size, max);
-		fatalx("exiting");
-	}
-}
-
-void
-iobuf_xfqueue(struct iobuf *io, const char *where, const char *fmt, ...)
+int
+io_xprintf(struct io *io, const char *fmt, ...)
 {
 	va_list	ap;
-	int	len;
+	int len;
 
 	va_start(ap, fmt);
-	len = iobuf_vfqueue(io, fmt, ap);
+	len = io_vprintf(io, fmt, ap);
 	va_end(ap);
+	if (len == -1)
+		fatal("io_xprintf(%p, %s, ...)", io, fmt);
 
-	if (len == -1) {
-		log_warnx("%s: iobuf_xfqueue(%p, %s, ...)", where, io, fmt);
-		fatalx("exiting");
-	}
+	return len;
+}
+
+int
+io_xprint(struct io *io, const char *str)
+{
+	int len;
+
+	len = io_print(io, str);
+	if (len == -1)
+		fatal("io_xprint(%p, %s, ...)", io, str);
+
+	return len;
 }
 #endif
 
@@ -457,9 +462,9 @@ mailaddr_match(const struct mailaddr *maddr1, const struct mailaddr *maddr2)
 
 	if (m2.user[0]) {
 		/* if address from table has a tag, we must respect it */
-		if (strchr(m2.user, TAG_CHAR) == NULL) {
+		if (strchr(m2.user, *env->sc_subaddressing_delim) == NULL) {
 			/* otherwise, strip tag from session address if any */
-			p = strchr(m1.user, TAG_CHAR);
+			p = strchr(m1.user, *env->sc_subaddressing_delim);
 			if (p)
 				*p = '\0';
 		}
@@ -790,3 +795,35 @@ base64_decode(char const *src, unsigned char *dest, size_t destsize)
 {
 	return __b64_pton(src, dest, destsize);
 }
+
+void
+log_trace(int mask, const char *emsg, ...)
+{
+	va_list	 ap;
+
+	if (tracing & mask) {
+		va_start(ap, emsg);
+		vlog(LOG_DEBUG, emsg, ap);
+		va_end(ap);
+	}
+}
+
+void
+log_trace_verbose(int v)
+{
+	tracing = v;
+
+	/* Set debug logging in log.c */
+	log_setverbose(v & TRACE_DEBUG ? 2 : foreground_log);
+}
+
+void
+xclosefrom(int lowfd)
+{
+#if defined HAVE_CLOSEFROM_INT
+    if (closefrom(lowfd) == -1)
+        err(1, "closefrom");
+#else
+    closefrom(lowfd);
+#endif
+}
